--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local Type, Version = 'BisonHijack', 1
local AceGUI = LibStub( 'AceGUI-3.0')
local WEAPON = { 'MH', 'OH', 'TH' }

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Control_OnMouseDown( frame, ...)
	frame.obj:Fire( 'OnMouseDown', ...)
end

local function Control_OnMouseUp( frame)
	frame.obj:Fire( 'OnMouseUp')
end

local function FreeBuff( self)
	local points = self.userdata.points
	local buff = self.userdata.buff
	if buff then
		buff:ClearAllPoints()
		for _,v in pairs( points) do
			buff:SetPoint( unpack( v))
		end
	end
end

local function BindBuff( self)
	local buff = self.userdata.buff
	if buff then
		local points = self.userdata.points
		for k in pairs( points) do
			points[k] = nil
		end
		for i = 1,buff:GetNumPoints() do
			tinsert( points, { buff:GetPoint( i)})
		end
	end
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	['OnAcquire'] = function( self)
		self.userdata.points = {}
	end,

	['OnRelease'] = function( self)
		FreeBuff( self)
	end,

	['SetBuff'] = function( self, buff, hijack)
		if not buff then
			FreeBuff( self)
		end
		self.userdata.buff = buff
		if hijack then
			BindBuff( self)
		end
		if buff then
			buff:ClearAllPoints()
			buff:SetAllPoints( self.frame)
		end
	end,

	['SetScale'] = function( self, scale)
		self.frame:SetScale( scale)
		if buff then
			buff:SetScale( scale)
		end
	end,

	['SetSpell'] = function( self, id)
--		if id then
--			self.spell:SetText( id)
--			self.spell:Show()
--		else
--			self.spell:Hide()
--		end
	end,

	['SetLock'] = function( self, locked)
		if self.userdata.locked ~= locked then
			self.userdata.locked = locked
			local frame = self.frame
			if locked then
				frame:SetScript( 'OnMouseDown', nil)
				frame:SetScript( 'OnMouseUp',   nil)
				frame:Hide()
--				self.ghost:Hide()
--				self.back:Hide()
			else
				frame:SetScript( 'OnMouseDown', Control_OnMouseDown)
				frame:SetScript( 'OnMouseUp',   Control_OnMouseUp)
				frame:Show()
--				self.ghost:Show()
--				self.back:Show()
			end
		end
	end,

	['Initialize'] = function( self, index, filter)
		if filter then
			self.ghost:SetText( index)
		else
			self.ghost:SetText( WEAPON[index] or '??')
		end
	end,

	['SetVisible'] = function( self, visible)
	end
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local id  = AceGUI:GetNextWidgetNum( Type)
	local name = Type .. id
	
	local frame = CreateFrame( 'Button', name, UIParent)
	frame:SetAlpha( 1)
	frame:SetWidth( 32)
	frame:SetHeight( 32)
	frame:SetFrameStrata( 'LOW')
	frame:Hide()
	
	local ghost = frame:CreateFontString( nil, 'OVERLAY', 'GameFontHighlight')
	ghost:SetHeight( 10)
	ghost:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	ghost:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
--	ghost:Hide()

	local back = frame:CreateTexture( nil, 'OVERLAY')
	back:SetTexture( 0, 0, 1, 1)
	back:SetBlendMode( 'ADD')
	back:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	back:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
--	back:Hide()

	local widget = {
		num      = id,
		frame    = frame,
		ghost    = ghost,
		back     = back,
		type     = Type
	}
	for method, func in pairs( methods) do
		widget[method] = func
	end
	frame.obj = widget
	
	return AceGUI:RegisterAsWidget( widget)
end

AceGUI:RegisterWidgetType( Type, Constructor, Version)
